@@ -1,8 +1,7 @@
-#! /bin/sh
-# -*- tcl -*- \
-exec tclsh "$0" ${1+"$@"}
+#! /usr/bin/env tclsh
+## -*- tcl -*-
 
-package require Tcl 8.4
+package require Tcl 8.4-
 package require Tk
 package require Plotchart
 
