@@ -1,14 +1,18 @@
-#!/usr/bin/env wish
+#! /usr/bin/env tclsh
 
 #==============================================================================
 # Demonstrates how to use a tablelist widget for displaying the content of a
-# directory.
+# directory.  Uses images based on the following icons:
 #
-# Copyright (c) 2010-2019  Csaba Nemethi (E-mail: csaba.nemethi@t-online.de)
+#   https://icons8.com/icon/JXYalxb9XWWd/folder
+#   https://icons8.com/icon/RdCT0UIsKOIp/opened-folder
+#   https://icons8.com/icon/mEF_vyjYlnE3/file
+#
+# Copyright (c) 2010-2023  Csaba Nemethi (E-mail: csaba.nemethi@t-online.de)
 #==============================================================================
 
-package require Tk 8.3
-package require tablelist 6.8
+package require Tk
+package require tablelist
 
 #
 # Add some entries to the Tk option database
@@ -17,11 +21,17 @@ set dir [file dirname [info script]]
 source [file join $dir option.tcl]
 
 #
-# Create three images
+# Create three images corresponding to the display's DPI scaling level
 #
-image create photo clsdFolderImg -file [file join $dir clsdFolder.gif]
-image create photo openFolderImg -file [file join $dir openFolder.gif]
-image create photo fileImg       -file [file join $dir file.gif]
+if {$tk_version >= 8.7 || [catch {package require tksvg}] == 0} {
+    set pct ""; set sfx "svg"; set fmt $tablelist::svgfmt
+} else {
+    set pct $tablelist::scalingpct; set sfx "gif"; set fmt "gif"
+}
+foreach name {clsdFolder openFolder file} {
+    set imgFile $name$pct.$sfx		;# e.g., "file.svg" or "file150.gif"
+    image create photo ${name}Img -file [file join $dir $imgFile] -format $fmt
+}
 
 #------------------------------------------------------------------------------
 # displayContents
@@ -54,6 +64,15 @@ proc displayContents dir {
     scrollbar $vsb -orient vertical   -command [list $tbl yview]
     scrollbar $hsb -orient horizontal -command [list $tbl xview]
 
+    #
+    # On X11 configure the tablelist according
+    # to the display's DPI scaling level
+    #
+    if {[tk windowingsystem] eq "x11"} {
+	global pct					;# ""|100|125|...|200
+	$tbl configure -treestyle bicolor$pct
+    }
+
     #
     # Create a pop-up menu with one command entry; bind the script
     # associated with its entry to the <Double-1> event, too
@@ -84,8 +103,7 @@ proc displayContents dir {
     # Manage the widgets
     #
     grid $tbl -row 0 -rowspan 2 -column 0 -sticky news
-    global winSys					;# see option.tcl
-    if {[string compare $winSys "win32"] == 0} {
+    if {[tk windowingsystem] eq "win32"} {
 	grid $vsb -row 0 -rowspan 2 -column 1 -sticky ns
     } else {
 	grid [$tbl cornerpath] -row 0 -column 1 -sticky ew
@@ -94,7 +112,7 @@ proc displayContents dir {
     grid $hsb -row 2 -column 0 -sticky ew
     grid rowconfigure    $tf 1 -weight 1
     grid columnconfigure $tf 0 -weight 1
-    pack $b1 $b2 $b3 -side left -expand yes -pady 10
+    pack $b1 $b2 $b3 -side left -expand yes -pady 7p
     pack $bf -side bottom -fill x
     pack $tf -side top -expand yes -fill both
 
@@ -116,15 +134,14 @@ proc putContents {dir tbl nodeIdx} {
     # The following check is necessary because this procedure
     # is also invoked by the "Refresh" and "Parent" buttons
     #
-    if {[string compare $dir ""] != 0 &&
-	(![file isdirectory $dir] || ![file readable $dir])} {
+    if {$dir ne "" && (![file isdirectory $dir] || ![file readable $dir])} {
 	bell
-	if {[string compare $nodeIdx "root"] == 0} {
+	if {$nodeIdx eq "root"} {
 	    set choice [tk_messageBox -title "Error" -icon warning -message \
 			"Cannot read directory \"[file nativename $dir]\"\
 			-- replacing it with nearest existent ancestor" \
 			-type okcancel -default ok]
-	    if {[string compare $choice "ok"] == 0} {
+	    if {$choice eq "ok"} {
 		while {![file isdirectory $dir] || ![file readable $dir]} {
 		    set dir [file dirname $dir]
 		}
@@ -136,8 +153,8 @@ proc putContents {dir tbl nodeIdx} {
 	}
     }
 
-    if {[string compare $nodeIdx "root"] == 0} {
-	if {[string compare $dir ""] == 0} {
+    if {$nodeIdx eq "root"} {
+	if {$dir eq ""} {
 	    if {[llength [file volumes]] == 1} {
 		wm title . "Contents of the File System"
 	    } else {
@@ -160,7 +177,7 @@ proc putContents {dir tbl nodeIdx} {
     # sorting purposes (it will be removed by formatString).
     #
     set itemList {}
-    if {[string compare $dir ""] == 0} {
+    if {$dir eq ""} {
 	foreach volume [file volumes] {
 	    lappend itemList [list D[file nativename $volume] -1 D $volume]
 	}
@@ -192,7 +209,7 @@ proc putContents {dir tbl nodeIdx} {
     #
     foreach item $itemList {
 	set name [lindex $item end]
-	if {[string compare $name ""] == 0} {			;# file
+	if {$name eq ""} {					;# file
 	    $tbl cellconfigure $row,0 -image fileImg
 	} else {						;# directory
 	    $tbl cellconfigure $row,0 -image clsdFolderImg
@@ -210,18 +227,18 @@ proc putContents {dir tbl nodeIdx} {
 	incr row
     }
 
-    if {[string compare $nodeIdx "root"] == 0} {
+    if {$nodeIdx eq "root"} {
 	#
 	# Configure the "Refresh" and "Parent" buttons
 	#
 	.bf.b1 configure -command [list refreshView $dir $tbl]
 	set b2 .bf.b2
-	if {[string compare $dir ""] == 0} {
+	if {$dir eq ""} {
 	    $b2 configure -state disabled
 	} else {
 	    $b2 configure -state normal
 	    set p [file dirname $dir]
-	    if {[string compare $p $dir] == 0} {
+	    if {$p eq $dir} {
 		$b2 configure -command [list putContents "" $tbl root]
 	    } else {
 		$b2 configure -command [list putContents $p $tbl root]
@@ -285,7 +302,9 @@ proc expandCmd {tbl row} {
 # tablelist widget tbl.
 #------------------------------------------------------------------------------
 proc collapseCmd {tbl row} {
-    $tbl cellconfigure $row,0 -image clsdFolderImg
+    if {[$tbl hasrowattrib $row pathName]} {		;# directory item
+	$tbl cellconfigure $row,0 -image clsdFolderImg
+    }
 }
 
 #------------------------------------------------------------------------------
@@ -386,8 +405,7 @@ proc restoreExpandedStates {tbl nodeIdx expandedFoldersName} {
 
     foreach key [$tbl childkeys $nodeIdx] {
 	set pathName [$tbl rowattrib $key pathName]
-	if {[string compare $pathName ""] != 0 &&
-	    [info exists expandedFolders($pathName)]} {
+	if {$pathName ne "" && [info exists expandedFolders($pathName)]} {
 	    $tbl expand $key -partly
 	    restoreExpandedStates $tbl $key expandedFolders
 	}
