@@ -1,12 +1,13 @@
-#!/usr/bin/env wish
+#! /usr/bin/env tclsh
 
 #==============================================================================
 # Demonstrates the use of embedded windows in tablelist widgets.
 #
-# Copyright (c) 2004-2019  Csaba Nemethi (E-mail: csaba.nemethi@t-online.de)
+# Copyright (c) 2004-2024  Csaba Nemethi (E-mail: csaba.nemethi@t-online.de)
 #==============================================================================
 
-package require tablelist_tile 6.8
+package require Tk
+package require tablelist_tile
 
 wm title . "Tile Library Scripts"
 
@@ -19,21 +20,21 @@ source [file join $dir option_tile.tcl]
 #
 # Create the font TkFixedFont if not yet present
 #
-catch {font create TkFixedFont -family Courier -size -12}
+catch {font create TkFixedFont -family Courier -size 9}
 
 #
-# Create an image to be displayed in buttons embedded in a tablelist widget
+# Create an image corresponding to the display's DPI scaling
+# level, to be displayed in buttons embedded in a tablelist widget
 #
-image create photo openImg -file [file join $dir open.gif]
-
-if {[tablelist::getCurrentTheme] eq "aqua"} {
-    #
-    # Work around the improper appearance of the tile scrollbars
-    #
-    if {[package vcompare $::tk_patchLevel "8.6.10"] < 0} {
-	interp alias {} ttk::scrollbar {} ::scrollbar
-    }
+if {$tk_version >= 8.7 || [catch {package require tksvg}] == 0} {
+    set fmt $tablelist::svgfmt
+    image create photo viewImg -file [file join $dir view.svg] -format $fmt
 } else {
+    set pct $tablelist::scalingpct
+    image create photo viewImg -file [file join $dir view$pct.gif] -format gif
+}
+
+if {$currentTheme ne "aqua"} {
     #
     # Make the embedded buttons as small as possible.  Recall that in most
     # themes, the tile buttons consist of the following element hierarchy:
@@ -72,7 +73,8 @@ if {[$tbl cget -selectborderwidth] == 0} {
     $tbl configure -spacing 1
 }
 $tbl columnconfigure 0 -name fileName
-$tbl columnconfigure 1 -formatcommand emptyStr -sortmode integer
+$tbl columnconfigure 1 -formatcommand emptyStr -sortmode integer \
+    -stretchwindow yes
 $tbl columnconfigure 2 -name fileSize -sortmode integer
 $tbl columnconfigure 4 -name seen
 ttk::scrollbar $vsb -orient vertical -command [list $tbl yview]
@@ -85,7 +87,7 @@ proc emptyStr val { return "" }
 set tblFont [$tbl cget -font]
 set size [font actual $tblFont -size]
 if {$size == 0} {					;# e.g., on Ubuntu
-    set size -12
+    set size 9
 }
 eval font create BoldFont [font actual $tblFont] -size $size -weight bold
 
@@ -111,6 +113,19 @@ foreach fileName [lsort [glob *.tcl]] {
 if {$tk_version >= 8.5} {
     $tbl header insert 0 [list "[$tbl size] *.tcl files" "" $totalSize "" ""]
     $tbl header rowconfigure 0 -foreground blue
+
+    if {$currentTheme eq "awdark" || $currentTheme eq "awbreezedark" ||
+	$currentTheme eq "black" || $currentTheme eq "breeze-dark" ||
+	$currentTheme eq "sun-valley-dark"} {
+	$tbl header rowconfigure 0 -foreground SkyBlue
+    } elseif {$currentTheme eq "aqua" &&
+	[package vcompare $tk_patchLevel "8.6.10"] >= 0} {
+	if {[tk::unsupported::MacWindowStyle isdark .]} {
+	    $tbl header rowconfigure 0 -foreground SkyBlue
+	}
+	bind . <<LightAqua>> { $tbl header rowconfigure 0 -foreground blue }
+	bind . <<DarkAqua>>  { $tbl header rowconfigure 0 -foreground SkyBlue }
+    }
 }
 
 #------------------------------------------------------------------------------
@@ -125,7 +140,8 @@ proc createFrame {tbl row col w} {
     # Create the frame and replace the binding tag "Frame"
     # with "TablelistBody" in the list of its binding tags
     #
-    frame $w -width 102 -height 14 -background ivory -borderwidth 1 \
+    set height [expr {[font metrics $::tblFont -linespace] * 9 / 10}]
+    frame $w -width 72p -height $height -background ivory -borderwidth 1 \
 	     -relief solid
     bindtags $w [lreplace [bindtags $w] 1 1 TablelistBody]
 
@@ -133,14 +149,14 @@ proc createFrame {tbl row col w} {
     # Create the child frame and replace the binding tag "Frame"
     # with "TablelistBody" in the list of its binding tags
     #
-    frame $w.f -height 12 -background red -borderwidth 1 -relief raised
+    frame $w.f -background red -borderwidth 1 -relief raised
     bindtags $w.f [lreplace [bindtags $w] 1 1 TablelistBody]
 
     #
     # Manage the child frame
     #
     set fileSize [$tbl cellcget $row,fileSize -text]
-    place $w.f -relwidth [expr {double($fileSize) / $::maxSize}]
+    place $w.f -relheight 1.0 -relwidth [expr {double($fileSize) / $::maxSize}]
 }
 
 #------------------------------------------------------------------------------
@@ -151,7 +167,7 @@ proc createFrame {tbl row col w} {
 #------------------------------------------------------------------------------
 proc createButton {tbl row col w} {
     set key [$tbl getkeys $row]
-    ttk::button $w -style Embedded.TButton -image openImg -takefocus 0 \
+    ttk::button $w -style Embedded.TButton -image viewImg -takefocus 0 \
 		   -command [list viewFile $tbl $key]
 }
 
@@ -180,9 +196,13 @@ proc viewFile {tbl key} {
     ttk::frame $tf -class ScrollArea
     set txt $tf.txt
     set vsb $tf.vsb
-    text $txt -background white -font TkFixedFont -setgrid yes \
-	      -yscrollcommand [list $vsb set]
-    catch {$txt configure -tabstyle wordprocessor}	;# for Tk 8.5 and above
+    text $txt -font TkFixedFont -setgrid yes -yscrollcommand [list $vsb set]
+    catch {$txt configure -tabstyle wordprocessor}
+    global isAwTheme
+    if {$isAwTheme} {
+	global currentTheme
+	ttk::theme::${currentTheme}::setTextColors $txt
+    }
     ttk::scrollbar $vsb -orient vertical -command [list $txt yview]
 
     #
@@ -203,7 +223,7 @@ proc viewFile {tbl key} {
     grid $vsb -row 0 -column 1 -sticky ns
     grid rowconfigure    $tf 0 -weight 1
     grid columnconfigure $tf 0 -weight 1
-    pack $btn -pady 10
+    pack $btn -pady 7p
     pack $bf -side bottom -fill x
     pack $tf -side top -expand yes -fill both
 
@@ -219,11 +239,8 @@ proc viewFile {tbl key} {
 #
 # Create embedded windows in the columns no. 1 and 3
 #
-set rowCount [$tbl size]
-for {set row 0} {$row < $rowCount} {incr row} {
-    $tbl cellconfigure $row,1 -window createFrame -stretchwindow yes
-    $tbl cellconfigure $row,3 -window createButton
-}
+$tbl fillcolumn 1 -window createFrame
+$tbl fillcolumn 3 -window createButton
 
 set bf .bf
 ttk::frame $bf
@@ -241,6 +258,6 @@ if {[tk windowingsystem] eq "win32"} {
 }
 grid rowconfigure    $tf 1 -weight 1
 grid columnconfigure $tf 0 -weight 1
-pack $btn -pady 10
+pack $btn -pady 7p
 pack $bf -side bottom -fill x
 pack $tf -side top -expand yes -fill both
